#ifndef TRACER_H
#define TRACER_H

#if defined(NDEBUG)
/* Release version: do nothing */

#define TR_MODULE(name)
#define TR_SET_CONFIG(xml_node)
#define TR_GET_CONFIG(xml_node)
#define TR_SHOW()
#define TR_HIDE()
#define TR_ENTER()
#define TR_LEAVE()
#define TR_FUNC()
#define TR_RET(value) return value
#define TR_RETURN() return
#define TR

#else
/* Debug version */

#include "TraceManager.h"

// Not: just including this header file will automatically create the following
// variable in a sourcefile. Therefor do not include this file in a generic header file,
// as it may result in a logging object in ALL files, which may not be what you want.
// Especially since it's dependant on Qt

#if defined(__GNUC__)
#define FUNCNAME __PRETTY_FUNCTION__
#elif defined(_MSC_VER)
  #if (_MSC_VER >= 1300)
  #define FUNCNAME __FUNCDNAME__
  #else
  #define FUNCNAME __FILE__,__LINE__
  #endif
#else
#error Compiler not supported (neither GNU C or Microsoft Visual Studio)
#endif

#include "TraceFunction.h"

#define TR_MODULE(name) static NTracer::CTraceModule *s_trace_module = NTracer::CTraceManager::Instance()->RegisterModule(name)

#define TR_SET_CONFIG(xml_node) NTracer::CTraceManager::Instance()->SetConfiguration(xml_node)
#define TR_GET_CONFIG(xml_node) NTracer::CTraceManager::Instance()->GetConfiguration(xml_node)

#define TR_SHOW()   NTracer::CTraceManager::Instance()->Show()
#define TR_HIDE()   NTracer::CTraceManager::Instance()->Hide()

#define TR_ENTER()  s_trace_module->Enter(FUNCNAME)
#define TR_LEAVE()  s_trace_module->Leave(FUNCNAME)
#define TR_FUNC     NTracer::TraceFunction TracerObject(s_trace_module, FUNCNAME); TracerObject

#define TR_RET(value)\
	{\
	s_trace_module->Leave(FUNCNAME);\
	return value;\
	}
#define TR_RETURN()\
	{\
	s_trace_module->Leave(FUNCNAME);\
	return;\
	}
#define TR s_trace_module->Log
#endif

#endif
